using System;
using System.Reflection;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Aurora.Core.EntityFramework;
using Microsoft.EntityFrameworkCore;
using Aurora.Core.Service;
using Aurora.Core.IService;
using Aurora.Infrastructure.Tenant;
using Aurora.Infrastructure.User;
using Aurora.Common.User;
using Aurora.Common.Tenant;
using Microsoft.AspNetCore.Http;

namespace Aurora.Core.Api
{
    public static class ModuleCoreApi
    {
        public static IServiceCollection AddModuleCoreApi(this IServiceCollection services, IConfiguration configuration)
        {
            if (services == null)
            {
                throw new ArgumentNullException("services");
            }

            services.AddAutoMapper(Assembly.Load("Aurora.Core.Service"));

            //注入请求上下文
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            services.AddScoped<ICurrentUserProvider, CurrentUserProvider>();
            services.AddScoped<ICurrentTenantProvider, CurrentTenantProvider>();

            services.AddDbContext<ApplicationDbContext>(option =>
                    option.UseMySQL(configuration["ConnectionStrings:MySql"]));
            services.AddDbContext<ApplicationReadonlyDbContext>(option =>
                    option.UseMySQL(configuration["ReadOnlyConnectionStrings:MySql"]));

            services.AddTransient<ICategoryService, CategoryService>();

            // var RFCOptions = services.BuildServiceProvider().GetService<IOptions<SAPRFCSettings>>();

            // //注入RFC链接
            // services.AddSingleton(p =>
            //    new RfcConnection(builder => builder
            //         .UseConnectionHost(RFCOptions.Value.HostName)
            //         .UseLogonUserName(RFCOptions.Value.UserName)
            //         .UseLogonPassword(RFCOptions.Value.Password)
            //         .UseLogonClient(RFCOptions.Value.Client)
            //         .UseLogonLanguage(RFCOptions.Value.LogonLanguage)
            //         .UseSystemNumber(RFCOptions.Value.SystemNumber)
            //         .UseSapRouter(RFCOptions.Value.SapRouter))
            // );

            // services.AddSingleton<MMGoodsBomClient, MMGoodsBomClient>();
            // services.AddSingleton<MMGoodsClient, MMGoodsClient>();
            // services.AddSingleton<MMImporrPostDataClient, MMImporrPostDataClient>();
            // services.AddSingleton<MMPOClient, MMPOClient>();
            // services.AddSingleton<SDExporrPostDataClient, SDExporrPostDataClient>();
            // services.AddSingleton<SDSOClient, SDSOClient>();
            return services;
        }

    }
}
