



using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using Aurora.Common.Entities;
using Aurora.Core.Domain.Entities;
using Aurora.Infrastructure.Tenant;
using Aurora.Infrastructure.User;
using Microsoft.EntityFrameworkCore;

namespace Aurora.Core.EntityFramework
{
    public class ApplicationDbContext : DbContext
    {
        private readonly ICurrentUser currentUser;
        private readonly ICurrentTenant currentTenant;

        public ApplicationDbContext(DbContextOptions<ApplicationDbContext> options) : base(options)
        {
            // ,
            // ICurrentUserProvider currentUserProvider,
            // ICurrentTenantProvider currentTenantProvider
            // currentUser = currentUserProvider.GetCurrentUser();
            // currentTenant = currentTenantProvider.GetCurrentTenant();
        }

        public DbSet<Category> Categories { get; set; }

        protected override void OnModelCreating(ModelBuilder builder)
        {
            //配置正在运行的程序集里所有实体映射配置类
            builder.ApplyConfigurationsFromAssembly(Assembly.GetExecutingAssembly());

            //多租户过滤
            foreach (var type in GetBaseEntityTypes())
            {
                var method = SetGlobalQueryMethod.MakeGenericMethod(type);
                method.Invoke(this, new object[] { builder });
            }

            // 配置表名映射
            builder.Entity<Category>().ToTable("category");
 
            base.OnModelCreating(builder);
        }

        private static IList<Type> _baseEntityTypesCache;

        private static IList<Type> GetBaseEntityTypes()
        {
            if (_baseEntityTypesCache != null)
                return _baseEntityTypesCache.ToList();
            _baseEntityTypesCache = (from t in typeof(TenantEntityBase).GetTypeInfo().Assembly.DefinedTypes
                                     where t.BaseType == typeof(TenantEntityBase)
                                     select t.AsType()).ToList();
            return _baseEntityTypesCache;
        }
        protected static readonly MethodInfo SetGlobalQueryMethod = typeof(ApplicationDbContext).GetMethods(BindingFlags.Public | BindingFlags.Instance)
                                                        .Single(t => t.IsGenericMethod && t.Name == "SetGlobalQuery");

        public void SetGlobalQuery<T>(ModelBuilder builder) where T : TenantEntityBase
        {
            builder.Entity<T>().HasQueryFilter(e => e.TenantCode == currentTenant.TenantCode);
        }
    }
}